/* $Workfile: BurstStatusOtxDevices.c $       $Revision: 5 $      $Modtime: 4/04/13 9:18a $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver Test Program

  @doc PublicDemo

  @module

    BurstStatusOtxDevices.C

    A simple example console application demonstrating the use of OTX Driver
    with the OTX boards.

		The application enumerates board serial number and available
    physical devices.
		
    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    Press 'h' or '?' for help. 'q' to quit.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <assert.h>
#include <time.h>

#include "OsaStd.h"
#include "OsaCon.h"
#include "OtxErr.h"
#include "OtxBrd.h"
#include "OtxDrv.h"
#include "OtxHdlc.h"
#include "OsaThrd.h"
#include "OtxBurst.h"
#include "OtxBurstD.h"

#define DEMO_DEFAULT_LOG_FILE "BurstStatusOtxDevices.log"
#define DEMO_DEFAULT_BOARD_NO 0
#define DEMO_MAX_BURST_DEVICE 20 // Most PC can support up to 6 PCI cards

typedef struct DemoEnvT {
  char           *pszInputStr;
  OTX_BOOL        bExitOnError;
  char            cwd[OTX_MAX_PATH];
  char            logFileName[OTX_MAX_PATH];
  int             boardNo;
} DemoEnvT;

static int ms_nIndentLevel = 0;

/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void ReadSerialNum(OTX_HANDLE hParentDevice, OTX_HANDLE hChildDevice, OTX_INT32 eDevType);
static void EnumerateDevices(OTX_HANDLE hParentDevice);
static void DemoDisplayBurstStats(OTX_UINT32 nBurstDeviceCnt);
static const char *DemoGetDeviceTypeString(IN OTX_HANDLE eDevType);
static signed int DemoGetDeviceNo(IN OTX_HANDLE hDevice);
static void DemoLogIndent();

static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);
static void DemoHelpPanel(void);
static void DemoLogf(const char *formatStr, ...);
static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT otxRc);
static void DemoExitQuestion(void);

static OTX_INT32 DemoProcessUserCommand(char commandChar);

/*******************************************************************************
 Global Data
*******************************************************************************/
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;

OTX_HANDLE  g_hBurstDevice[DEMO_MAX_BURST_DEVICE];
OTX_UINT32  g_nBurstDeviceCnt = 0;
char       *g_pszSerialNumber[DEMO_MAX_BURST_DEVICE];


/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->pszInputStr       = NULL;
  envP->bExitOnError      = OTX_FALSE;
  envP->boardNo           = DEMO_DEFAULT_BOARD_NO;

  strcpy(envP->logFileName, envP->cwd);
  strcat(envP->logFileName, DEMO_DEFAULT_LOG_FILE);
  for (i = 1; i < argc; i++) {									// Scan command line arguments
    if (*(argv[i]+1) == 'Z') {   // Command input string -- case sensitive
      envP->pszInputStr = argv[i]+2;
    } else {    
      OsaStdStrUpr(argv[i]);
      if (*argv[i] == '-' || *argv[i] == '/') {		// Is argument a legal switch?
        if (*(argv[i]+1) == 'L') {								// Log file
          strcpy(envP->logFileName, argv[i] + 2);
        } else if (*(argv[i]+1) == 'Q') {   // Exit on error
          envP->bExitOnError = OTX_TRUE;
        } else if (*(argv[i]+1) == 'B') {					// Board number
          envP->boardNo = strtoul(argv[i]+2, NULL, 0);
        } else {
          DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
				  exit(1);
        }
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
			  exit(1);
      }
    }
  }

}


/*******************************************************************************

  @func

  Displays a help for the command line.

  @comm

*******************************************************************************/
static void DemoHelpPanel()
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /B<boardNo>  : Board number (Default is %d)\n", DEMO_DEFAULT_BOARD_NO);
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /Z<str>      : Command string (overrides keyboard input)\n");
  DemoLogf(" /Q           : Exit without prompting on errors\n");
}


/*******************************************************************************

  @func

  Displays a message on the screen and prints it to the logfile.

  @comm

*******************************************************************************/
static void DemoLogf(const char *formatStr, ...)
{
  va_list argList;

  va_start(argList, formatStr);
  OsaConsolePrintF((char *) formatStr, argList);
  va_end(argList);

  if (g_pDemoLogFile) {
    va_start(argList, formatStr);
    vfprintf(g_pDemoLogFile,   formatStr, argList);
    va_end(argList);
  }


}


static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  if ( OTX_FAILURE(result) ) {

    if (hDevice == OTX_INVALID_HANDLE_VALUE) {
      DemoLogf("ERROR(%#lX) \n", result);
    } else {
      OTX_RESULT otxRc;
      char msgStr[256];
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      OTX_UINT32      eDeviceType = 999;    // Device type for hDevice 
      OTX_UINT32      nDeviceNumber = 999;  // Device number for hDevice
      
      // Get the Device Type for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_TYPE, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_TYPE, &eDeviceType); 
      } 
      
      // Get the Device Number for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_NO, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDeviceNumber); 
      }
      
      // Convert the result code to a more descriptive string
      otxRc = OtxDrvResultCode2String(OTX_DRIVER_DEVICE_HANDLE, result, 256, &msgStr[0]);
      if (OTX_FAILURE(otxRc)) {
        strcpy(msgStr, "Cannot translate this result code.");
      }
      DemoLogf("ERROR(%#lX [%d.%d]) Dev(%X,%d,%d): %s \n", OTX_RESULT_CODE(result), OTX_RESULT_SOURCE_ID(result), OTX_RESULT_LINE_NUM(result), hDevice, (int) eDeviceType, (int)nDeviceNumber, msgStr);
    }

    DemoExitQuestion();
  }
}


/*******************************************************************************
  @func
    Prompts whether to exit the program. Usually called after an error 
    has been reported.

*******************************************************************************/
void DemoExitQuestion()
{
  short c;

  if (g_demoEnv.bExitOnError) {
    c = 'y';
  } else {
    DemoLogf("Do you want to exit (Y/N) ");
    c = OsaConsoleGetCh();
  }
  if (c == 'y' || c == 'Y') {
    // Clean up
    OtxDrvReleaseLib();
    OsaConsoleExit();
    exit(1);
  }
}



/*******************************************************************************

  @func

  Processes User Commands

  @comm

*******************************************************************************/
static OTX_INT32 DemoProcessUserCommand(char commandChar)
{
  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?   Display Help text\n");
    DemoLogf(" h   Ditto\n");
    DemoLogf(" q   Quit\n");
    DemoLogf(" a   Update Burst Device List\n");
    DemoLogf(" b   Display Burst Status\n");
    DemoLogf(" B   Display Burst Status One Device\n");
    DemoLogf(" d   Display List of Burst Device Found\n");
    DemoLogf(" r   Clear/reset burst status\n");
    break;

  case 'q':
  case 'Q':
    return 1;
    break;

  case 'a':
    {
      OtxVersionInfo sVersionInfo;
      char szVersionInfo[255];
      OTX_RESULT result;

      result = OtxDrvGetAttributeValueSz(0, OTX_ATTR_DRV_REV_STR, szVersionInfo, 255); DemoCheckRc(0, result);
      result = OtxDrvGetAttributeValue(0, OTX_ATTR_DRV_REV_NUM, &sVersionInfo, sizeof(OtxVersionInfo), NULL); DemoCheckRc(0, result);

      DemoLogf("Driver version string     %s\n", szVersionInfo);
      DemoLogf("Driver major version      %d\n", sVersionInfo.nMjRev);
      DemoLogf("Driver minor version      %d\n", sVersionInfo.nMnRev);
      DemoLogf("Driver status             %s\n", sVersionInfo.bRelStatus ? "Released" : "Preliminary");
      DemoLogf("Driver preliminary level  %d\n", sVersionInfo.nPrelLevel);

    }

    // reset the global values
    for(g_nBurstDeviceCnt = 0; g_nBurstDeviceCnt < DEMO_MAX_BURST_DEVICE; g_nBurstDeviceCnt++) {
      if(g_pszSerialNumber[g_nBurstDeviceCnt] != 0) {
        // Need to free previous string
        free(g_pszSerialNumber[g_nBurstDeviceCnt]);
      }
      g_pszSerialNumber[g_nBurstDeviceCnt] = 0; 
      g_hBurstDevice[g_nBurstDeviceCnt] = OTX_INVALID_HANDLE_VALUE;
    }
    g_nBurstDeviceCnt = 0;

    EnumerateDevices(0);
    break;

  case 'B':
    {
      OTX_UINT32 nBurstDeviceCnt;
      for(nBurstDeviceCnt=0; nBurstDeviceCnt < g_nBurstDeviceCnt; nBurstDeviceCnt++) {
        DemoLogf("\n");
        DemoDisplayBurstStats(nBurstDeviceCnt);
      }
    }
    break;

  case 'b':
    {
      static OTX_UINT32 nBurstDeviceCnt;
      char strBuf[16];

      DemoLogf("\n");

      DemoLogf("Enter burst device number (%d): ", nBurstDeviceCnt);
      OsaConsoleGetStr(strBuf);

      if (strBuf[0]) {
        nBurstDeviceCnt = strtol(strBuf, NULL, 0);
      }
      
      if(nBurstDeviceCnt < g_nBurstDeviceCnt) {
        DemoLogf("Device Number %d \n", nBurstDeviceCnt);

        DemoLogf("\n");
        DemoDisplayBurstStats(nBurstDeviceCnt);
      }
      else {
        DemoLogf("Device Number out of range %d, should be less then %d\n", 
                 nBurstDeviceCnt, 
                 nBurstDeviceCnt);
      }

      DemoLogf("\n");

    }
    break;

  case 'd': // display the list of burst devices
    {
      OTX_UINT32 nBurstDeviceCnt;
      for(nBurstDeviceCnt=0; nBurstDeviceCnt < g_nBurstDeviceCnt; nBurstDeviceCnt++) {
        DemoLogf("\n");
        if(g_pszSerialNumber[nBurstDeviceCnt]) {
          DemoLogf("Burst Device %d Board Serial Number %s\n", 
                   nBurstDeviceCnt,
                   g_pszSerialNumber[nBurstDeviceCnt]);
        } 
        else {
          DemoLogf("\nBoard Serial Number String set to NULL\n");
        }
      }
    }
    break;

    case 'r': // Clear/reset burst status
    {
      OTX_UINT32 nBurstDeviceCnt;
      for(nBurstDeviceCnt=0; nBurstDeviceCnt < g_nBurstDeviceCnt; nBurstDeviceCnt++) {
        DemoLogf("Clear Burst Status %d\n", nBurstDeviceCnt);
        if (g_hBurstDevice[nBurstDeviceCnt] != OTX_INVALID_HANDLE_VALUE) {
           OtxDrvDbgDump(g_hBurstDevice[nBurstDeviceCnt], 101);   // reset stats
        }
      }
    }
    break;

  case '~':
    {
      char strBuf[12];
      OTX_INT32 nSleepTime;

      DemoLogf("Enter milliseconds to sleep :");
      OsaConsoleGetStr(strBuf);

      if (strBuf[0]) {
        nSleepTime = strtol(strBuf, NULL, 0);
        DemoLogf("Sleeping %d milliseconds\n", nSleepTime);
        OsaThreadSleep(nSleepTime);
      }
    }
    break;

  case 0xD: // CR
    // No command just return
    DemoLogf("\n");
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}


/*******************************************************************************

  @func

    Returns the state of the Burst device (enabled or disabled).

*******************************************************************************/
OTX_BOOL DemoIsBurstDeviceEnabled(
                                  OTX_UINT32 nBurstDeviceCnt
                                 )
{
  OTX_RESULT result = OTX_S_OK;
  OtxDeviceStateE  eDevState;
  OTX_HANDLE       hBurstDevice;

  if(g_hBurstDevice[nBurstDeviceCnt] == OTX_INVALID_HANDLE_VALUE) {
    // No Burst device set
    DemoLogf("\nNo Burst Device Found\n");
    return OTX_FALSE;
  }
  else {
    hBurstDevice = g_hBurstDevice[nBurstDeviceCnt];
  } 

  result = OtxDrvGetState(hBurstDevice, &eDevState); 
  DemoCheckRc(hBurstDevice, result);

  if (eDevState == OTX_DEVICE_WORKING) {
    return OTX_TRUE; // The burst device is enabled
  } else {
    return OTX_FALSE; // The burst device is not enabled
  }
}


/*******************************************************************************

  @func

    Read board serial number

*******************************************************************************/
static void ReadSerialNum(
                          OTX_HANDLE hParentDevice, 
                          OTX_HANDLE hChildDevice, 
                          OTX_INT32 eDevType
                          )
{
  OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
	OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
	OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
	OTX_RESULT      result;
	char *pszSerialNumber;
  char *pszBrdBuildNumber;
  char *pszBrdUserString;
  char *pszCpldBuildNumbers;
	OTX_UINT32      nValue;  
				
  if( hParentDevice == 0 && eDevType != -1)		// Parent device with children
	{
    result = OtxDrvGetAttributeType(hChildDevice, OTX_ATTR_BRD_SERIAL_NUMBER, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(hChildDevice, result);
		assert(eAttribType == OTX_ATTRIB_SZ);
		assert(bReadOnly == OTX_TRUE);
		pszSerialNumber = (char *) malloc(cbAttribValue * sizeof(char));
    if(g_pszSerialNumber[g_nBurstDeviceCnt] != 0) {
      // There should be only one Busrt device per board
		  DemoLogf("\nGlobal board serial number already set\n");
    }
    g_pszSerialNumber[g_nBurstDeviceCnt] = pszSerialNumber; 
		result = OtxDrvGetAttributeValueSz(hChildDevice, OTX_ATTR_BRD_SERIAL_NUMBER, pszSerialNumber, cbAttribValue); DemoCheckRc(hChildDevice, result);
		DemoLogf("\nBoard Serial Number: %s \n", pszSerialNumber);
		//free(pszSerialNumber) later will use the SN during status display

    result = OtxDrvGetAttributeType(hChildDevice, OTX_ATTR_BRD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(hChildDevice, result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszBrdBuildNumber = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(hChildDevice, OTX_ATTR_BRD_BUILD, pszBrdBuildNumber, cbAttribValue); DemoCheckRc(hChildDevice, result);
    DemoLogf("  Board-Build: %s", pszBrdBuildNumber);
    free(pszBrdBuildNumber);

    DemoLogf("\n");

    result = OtxDrvGetAttributeType(hChildDevice, OTX_ATTR_BRD_USER_STRING, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(hChildDevice, result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszBrdUserString = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(hChildDevice, OTX_ATTR_BRD_USER_STRING, pszBrdUserString, cbAttribValue); DemoCheckRc(hChildDevice, result);
    if (*pszBrdUserString) {
      DemoLogf("  Board-UserString: %s", pszBrdUserString);
      DemoLogf("\n");
    }
    free(pszBrdUserString);

    result = OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_BRD_DEVICE_ID, &nValue); DemoCheckRc(hChildDevice, result);
    DemoLogf("  DeviceId: %d", nValue);

    result = OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_BRD_SUBSYSTEM_ID, &nValue); DemoCheckRc(hChildDevice, result);
    DemoLogf("  SubSystemId: 0x%X", nValue);

    DemoLogf("\n");

    result = OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_BRD_PCI_DEVICE_NUMBER, &nValue); 
    if (OTX_RESULT_CODE(result) == OTX_S_OK) {
      DemoLogf("  PciDevNo: 0x%X", nValue);
    }

    result = OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_BRD_PCI_FUNCTION_NUMBER, &nValue); 
    if (OTX_RESULT_CODE(result) == OTX_S_OK) {
      DemoLogf("  PciFuncNo: 0x%X\n", nValue);
    }

    result = OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_BRD_PCMCIA_SOCKET_NUMBER, &nValue); 
    if (OTX_RESULT_CODE(result) == OTX_S_OK) {
      DemoLogf("  PcmciaSocketNo: 0x%X\n", nValue);
    }


    result = OtxDrvGetAttributeType(hChildDevice, OTX_ATTR_BRD_CPLD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); 
    if (OTX_RESULT_CODE(result) == OTX_S_OK) {
      assert(eAttribType == OTX_ATTRIB_SZ);
      assert(bReadOnly == OTX_TRUE);
      pszCpldBuildNumbers = (char *) malloc(cbAttribValue * sizeof(char));
      result = OtxDrvGetAttributeValueSz(hChildDevice, OTX_ATTR_BRD_CPLD_BUILD, pszCpldBuildNumbers, cbAttribValue); DemoCheckRc(hChildDevice, result);
      DemoLogf("  CPLD-Build:\n%s", pszCpldBuildNumbers);
      free(pszCpldBuildNumbers);
    }

    DemoLogf("\n");
	}

}


/*******************************************************************************

  @func

  Enumerates all devices for a given parent devices and prints attributes
  etc.

  @comm

*******************************************************************************/
static void EnumerateDevices(OTX_HANDLE hParentDevice)
{
  OTX_UINT32   eDevType;
  OTX_ITERATOR itrChildTypes;
  OTX_ITERATOR itrChildrenOfSameType;


  // This is a recursive function so we indent one more level for each recursion

  // Initialize the opaque iterator for the hosted types
  OtxDrvIteratorBegin(&itrChildTypes);
  while (OTX_SUCCESS(OtxDrvEnumDeviceType(hParentDevice, &itrChildTypes, (OTX_INT32 *)&eDevType))) {   // Note that we must use a zero handle to get the children of the root level
    OTX_HANDLE hChildDevice;
    // Initialize the opaque iterator for the devices within the same type
    OtxDrvIteratorBegin(&itrChildrenOfSameType);
    while (OTX_SUCCESS(OtxDrvEnumDevice(hParentDevice, eDevType, &itrChildrenOfSameType, &hChildDevice))) {
      if (hChildDevice != OTX_INVALID_HANDLE_VALUE) {
        OTX_UINT32 eDevType2;
        OTX_HANDLE hParentDevice2;

				// Here we have a handle for a child device, get the attributes
        DemoLogIndent();
        DemoLogf("%s_%d (%s, handle = %#x)\n", 
          DemoGetDeviceTypeString(hChildDevice), 
          DemoGetDeviceNo(hChildDevice), 
          (eDevType >= 0) ? (eDevType == 0 ? "UnKnown" : "Physical") : "Logical", 
          hChildDevice);

        // Verify that the type and host attributes correspond to the ones we have since earlier
        OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_DEVICE_TYPE, &eDevType2);
        OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_DEVICE_HOST, &hParentDevice2);
        if (eDevType2 != eDevType) {
          DemoLogIndent();DemoLogf("  Incorrect attribute: OTX_ATTR_DEVICE_TYPE\n");
        }
        if (hParentDevice2 != hParentDevice) {
          DemoLogIndent();DemoLogf("  Incorrect attribute: OTX_ATTR_DEVICE_HOST\n");
        }

        // Print device specific attributes
        switch (eDevType) {
        case OTX_DEVICE_BURST:
          // Need to save this handle to be used during status read
          if(g_nBurstDeviceCnt < DEMO_MAX_BURST_DEVICE) {

            DemoLogf("Burst device found, count %d, Device 0x%X\n", g_nBurstDeviceCnt, hChildDevice);

            g_hBurstDevice[g_nBurstDeviceCnt] = hChildDevice;
            g_nBurstDeviceCnt++; // One burst device per board
            if(g_nBurstDeviceCnt >= DEMO_MAX_BURST_DEVICE) {
              // Max array value reached
              g_nBurstDeviceCnt = DEMO_MAX_BURST_DEVICE - 1;
            }
          } 
          else {
            // Should not get here
            DemoLogf("Maximum Burst Device limit reached\n");
          }
          break;
        case OTX_LDEVICE_PIPE:
          {
            OTX_UINT32 nValue;
            OTX_UINT8  sqConnMask[OTX_ANY_PIPE_TIMESLOTS_MAX];
            unsigned int i;

            OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_PIPE_CAPACITY, &nValue);
            DemoLogIndent();DemoLogf("  Capacity = %d\n", nValue);

            OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_PIPE_IN_CONNECTED, &nValue);
            if (nValue != OTX_INVALID_HANDLE_VALUE) {
              DemoLogIndent();DemoLogf("  Input connected to device with handle = %#x (%s_%d)\n", nValue, DemoGetDeviceTypeString(nValue), DemoGetDeviceNo(nValue));
              OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_PIPE_IN_HIGHWAY, &nValue);
              DemoLogIndent();DemoLogf("  Input connected to highway = %d\n", nValue);
              OtxDrvGetAttributeValue(hChildDevice, OTX_ATTR_PIPE_IN_CONN_MASK, sqConnMask, OTX_ANY_PIPE_TIMESLOTS_MAX, &nValue);
              DemoLogIndent();DemoLogf("  Input connection mask = ");
              for (i = 0; i < nValue; i++) {
                DemoLogf(" %0.2x", sqConnMask[i]);
              }
              DemoLogf("\n");

            } else {
              DemoLogIndent();DemoLogf("  Input not connected\n");
            }

            OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_PIPE_OUT_CONNECTED, &nValue);
            if (nValue != OTX_INVALID_HANDLE_VALUE) {
              
              DemoLogIndent();DemoLogf("  Output connected to device with handle = %#x (%s_%d)\n", nValue, DemoGetDeviceTypeString(nValue), DemoGetDeviceNo(nValue));
              OtxDrvGetAttributeValueUint32(hChildDevice, OTX_ATTR_PIPE_OUT_HIGHWAY, &nValue);
              DemoLogIndent();DemoLogf("  Output connected to highway = %d\n", nValue);

              OtxDrvGetAttributeValue(hChildDevice, OTX_ATTR_PIPE_OUT_CONN_MASK, sqConnMask, OTX_ANY_PIPE_TIMESLOTS_MAX, &nValue);
              DemoLogIndent();DemoLogf("  Output connection mask = ");
              for (i = 0; i < nValue; i++) {
                DemoLogf(" %0.2x", sqConnMask[i]);
              }
              DemoLogf("\n");
            } else {
              DemoLogIndent();DemoLogf("  Output not connected\n");
            }

          }
          break;
        case OTX_LDEVICE_EVENT_QUEUE:
          break;
        case OTX_DEVICE_LI_POTS:
          break;
        case OTX_DEVICE_ARNI_PCI:     
        case OTX_DEVICE_BALDER_PCMCIA:
        case OTX_DEVICE_BALDER_S_PCI:
        case OTX_DEVICE_BALDER_U_PCI:
        case OTX_DEVICE_THOR_PCMCIA:
        case OTX_DEVICE_VIDAR_5x_PCI:
        case OTX_DEVICE_VIDAR_6x_PCI:
        case OTX_DEVICE_SIF_PCI:
        case OTX_DEVICE_THOR_PCI:
        case OTX_DEVICE_THOR_8_PCI:
        case OTX_DEVICE_GIMLE_16_PCI_PLUS:
        case OTX_DEVICE_GIMLE_16_PCIE:
        case OTX_DEVICE_THOR_1_PCI_BASIC:
        case OTX_DEVICE_THOR_8_PCI_PLUS:       
        case OTX_DEVICE_THOR_2_PCI_PLUS:       
        case OTX_DEVICE_SLEIPNIR_1_PCI:        
        case OTX_DEVICE_VIDAR_55X12_PCI:       
        case OTX_DEVICE_THOR_4_PCI_PLUS:       
        case OTX_DEVICE_THOR_1_PCI_PLUS:       
        case OTX_DEVICE_ARNI_6X6_PCI_PLUS:     
        case OTX_DEVICE_ARNI_6X6_PCI:          
        case OTX_DEVICE_ARNI_6_FXS_PCI_PLUS:   
        case OTX_DEVICE_ARNI_6_FXO_PCI_PLUS:   
        case OTX_DEVICE_ARNI_6X3_PCI_PLUS:     
        case OTX_DEVICE_HUGIN_2_PC104:         
        case OTX_DEVICE_GIMLE_16_PCI_BASIC_32: 
        case OTX_DEVICE_GIMLE_16_PCI_BASIC_64: 
        case OTX_DEVICE_THOR_1_PCI_EXPRESS:    
        case OTX_DEVICE_THOR_2_PCI_EXPRESS:
        case OTX_DEVICE_THOR_2_EXPRESSCARD:
        case OTX_DEVICE_THOR_4_EXPRESSCARD:
        case OTX_DEVICE_THOR_8M_USB:
        case OTX_DEVICE_THOR_8_USB:
          {
            // Read serial number from the board
            ReadSerialNum( hParentDevice, hChildDevice, eDevType );
          }
          break;
        }

        // Now do the same thing for all children of this device (recursion)
        ms_nIndentLevel += 1;
        EnumerateDevices(hChildDevice);
        ms_nIndentLevel -= 1;
      } 
      else {
        // This should not happen
        DemoLogf("Internal Error: queried a handle value but it still didn't work??? Should not happen!\n");
      }
    }
  }
}


/*******************************************************************************
  @func
    Print DMA statistics.

    These stats are obtained from the driver, and are being used during
    initial release as a debugging feature.  
*******************************************************************************/
void DemoDisplayBurstStats(
                           OTX_UINT32 nBurstDeviceCnt
                           )
{
  OTX_RESULT        result;
  OTX_UINT32        stat_Starts;            // Number of actual dma transfers initiated
  OTX_UINT32        stat_Completes;         // Number of actual dma transfers completed IRQ's
  OTX_UINT32        stat_Partial;           // Number of partial transfer due to framemask
  OTX_UINT32        stat_Full;              // Number of count of max dma size transfers
  OTX_UINT32        stat_Aborts;            // Number of actual dma transfers aborts
  OTX_UINT32        stat_Amount;            // Number of amount of data transfered
  OTX_UINT32        stat_Errors;            // Number of count of processing errors
  OTX_UINT32        stat_OUruns;            // Number of amount of over/under runs
  OTX_UINT32        stat_OUrunsUser;        // Number of amount of over/under runs caused by user mode
  OTX_UINT32        stat_DmaBusy;           // Number of count of DMA busy occurances
  OTX_UINT32        stat_Notify;            // Number of logical device notifies
  OTX_UINT32        stat_NotifyPartial;     // Number of logical device notifies - partial
  OTX_UINT32        stat_FifoSize;          // size of the Burst FIFO
  OTX_UINT32        stat_FifoTrigger;       // the current FIFO Trigger value
  OTX_UINT32        stat_FifoCount;         // the current amount in the FIFO
  OTX_UINT32        stat_FifoPeak;          // the peak FIFO usage
  OTX_UINT32        stat_FifoTriggerCount;  // the current FIFO trigger fire value
  OTX_UINT32        stat_CodeErrors;
  OTX_UINT32        stat_Reserved0;
  OTX_UINT32        stat_Reserved1;
  OTX_UINT32        stat_Reserved2;
  OTX_UINT32        stat_Reserved3;
  OTX_UINT32        stat_Reserved4;
  OTX_UINT32        stat_Reserved5;
  OTX_UINT32        stat_Reserved6;
  OTX_UINT32        stat_Reserved7;
  OTX_HANDLE        hBurstDevice;

#define STATS_SHOW_FRAME_REF
#ifdef STATS_SHOW_FRAME_REF
  OTX_UINT32                ui32;
  OtxBurstFrameRefAndTime   FrameRef;             // frame reference data
  static OTX_UINT64         nLastFrame = 0;
#endif  // #ifdef STATS_SHOW_FRAME_REF
  
  if(g_hBurstDevice[nBurstDeviceCnt] == OTX_INVALID_HANDLE_VALUE) {
    // No Burst device set
    DemoLogf("\nNo Burst Device Found\n");
    return;
  }
  else {
    hBurstDevice = g_hBurstDevice[nBurstDeviceCnt];
  } 

  if(!DemoIsBurstDeviceEnabled(nBurstDeviceCnt)) {
    DemoLogf("\nBurst Device Not Enabled\n");
    return;
  }

  if(g_pszSerialNumber[nBurstDeviceCnt]) {
    DemoLogf("\nBoard Serial Number %s\n", g_pszSerialNumber[nBurstDeviceCnt]);
  }

  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_STARTS,             &stat_Starts            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_COMPLETES,          &stat_Completes         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_PARTIAL,            &stat_Partial           ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_FULL,               &stat_Full              ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_ABORTS,             &stat_Aborts            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_AMOUNT,             &stat_Amount            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_ERRORS,             &stat_Errors            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_OURUNS,             &stat_OUruns            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_OURUNS_USER,        &stat_OUrunsUser        ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_DMABUSY,            &stat_DmaBusy           ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_NOTIFY,             &stat_Notify            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_NOTIFY_PARTIAL,     &stat_NotifyPartial     ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_RECEIVER_FIFO_SIZE,           &stat_FifoSize          ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_RECEIVER_FIFO_TRIGGER,        &stat_FifoTrigger       ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_RECEIVER_FIFO_COUNT,          &stat_FifoCount         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_RECEIVER_FIFO_PEAK,           &stat_FifoPeak          ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_RECEIVER_FIFO_TRIGGER_COUNT,  &stat_FifoTriggerCount  ); DemoCheckRc(hBurstDevice, result);

  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_CODE_ERRORS,        &stat_CodeErrors        ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED0,          &stat_Reserved0         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED1,          &stat_Reserved1         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED2,          &stat_Reserved2         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED3,          &stat_Reserved3         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED4,          &stat_Reserved4         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED5,          &stat_Reserved5         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED6,          &stat_Reserved6         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_RECV_RESERVED7,          &stat_Reserved7         ); DemoCheckRc(hBurstDevice, result);


  DemoLogf("Burst Recv:: \nStarts:%d, Completes:%d, Partial:%d \nFull:%d, Aborts:%d, Amount:%x \nErrors:%d, OUErrors:%d, OUErrUser:%d \nDMABusy:%d, Notify:%d, NotifyPartial:%d, FIFO Size:%d \nTrigger:%d, Count:%d, Peak:%d, TriggerCount:%d\n",
      stat_Starts,
      stat_Completes,
      stat_Partial,
      stat_Full,
      stat_Aborts,
      stat_Amount,
      stat_Errors,
      stat_OUruns,
      stat_OUrunsUser,
      stat_DmaBusy,
      stat_Notify,
      stat_NotifyPartial,
      stat_FifoSize,
      stat_FifoTrigger,
      stat_FifoCount,
      stat_FifoPeak,
      stat_FifoTriggerCount
  );
  DemoLogf("stat_CodeErrors:%d \nstat_Reserved0:%d, stat_Reserved1:%d, stat_Reserved2:%d \nstat_Reserved3:%d, stat_Reserved4:%x, stat_Reserved5:%d \nstat_Reserved6:%d, stat_Reserved7:%d\n",
      stat_CodeErrors,
      stat_Reserved0,
      stat_Reserved1,
      stat_Reserved2,
      stat_Reserved3,
      stat_Reserved4,
      stat_Reserved5,
      stat_Reserved6,
      stat_Reserved7
  );

  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_STARTS,             &stat_Starts            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_COMPLETES,          &stat_Completes         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_PARTIAL,            &stat_Partial           ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_FULL,               &stat_Full              ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_ABORTS,             &stat_Aborts            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_AMOUNT,             &stat_Amount            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_ERRORS,             &stat_Errors            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_OURUNS,             &stat_OUruns            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_OURUNS_USER,        &stat_OUrunsUser        ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_DMABUSY,            &stat_DmaBusy           ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_NOTIFY,             &stat_Notify            ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_NOTIFY_PARTIAL,     &stat_NotifyPartial     ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_SENDER_FIFO_SIZE,             &stat_FifoSize          ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_SENDER_FIFO_TRIGGER,          &stat_FifoTrigger       ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_SENDER_FIFO_COUNT,            &stat_FifoCount         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_SENDER_FIFO_PEAK,             &stat_FifoPeak          ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_SENDER_FIFO_TRIGGER_COUNT,    &stat_FifoTriggerCount  ); DemoCheckRc(hBurstDevice, result);

  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_CODE_ERRORS,        &stat_CodeErrors        ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED0,          &stat_Reserved0         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED1,          &stat_Reserved1         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED2,          &stat_Reserved2         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED3,          &stat_Reserved3         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED4,          &stat_Reserved4         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED5,          &stat_Reserved5         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED6,          &stat_Reserved6         ); DemoCheckRc(hBurstDevice, result);
  result = OtxDrvGetAttributeValueUint32(hBurstDevice, OTX_ATTR_BURST_STAT_SEND_RESERVED7,          &stat_Reserved7         ); DemoCheckRc(hBurstDevice, result);

  DemoLogf("\n");

  {
    OTX_BOOL bDisabled;
    result = OtxDrvGetAttributeValueBool(hBurstDevice, 
                                         OTX_ATTR_BURST_SEND_DISABLED,  
                                         &bDisabled);
    DemoCheckRc(hBurstDevice, result);
    if(bDisabled == OTX_TRUE) {
      DemoLogf("Burst Send:: Is disabled on this board\n");
    }
  }

  DemoLogf("Burst Send:: \nStarts:%d, Completes:%d, Partial:%d \nFull:%d, Aborts:%d, Amount:%x \nErrors:%d, OUErrors:%d, OUErrUser:%d \nDMABusy:%d, Notify:%d, NotifyPartial:%d \nFIFO Size:%d, Trigger:%d, Count:%d, Peak:%d, TriggerCount:%d\n",
      stat_Starts,
      stat_Completes,
      stat_Partial,
      stat_Full,
      stat_Aborts,
      stat_Amount,
      stat_Errors,
      stat_OUruns,
      stat_OUrunsUser,
      stat_DmaBusy,
      stat_Notify,
      stat_NotifyPartial,
      stat_FifoSize,
      stat_FifoTrigger,
      stat_FifoCount,
      stat_FifoPeak,
      stat_FifoTriggerCount
  );
  DemoLogf("stat_CodeErrors:%d \nstat_Reserved0:%d, stat_Reserved1:%d, stat_Reserved2:%d \nstat_Reserved3:%d, stat_Reserved4:%x, stat_Reserved5:%d \nstat_Reserved6:%d, stat_Reserved7:%d\n",
      stat_CodeErrors,
      stat_Reserved0,
      stat_Reserved1,
      stat_Reserved2,
      stat_Reserved3,
      stat_Reserved4,
      stat_Reserved5,
      stat_Reserved6,
      stat_Reserved7
  );

#ifdef STATS_SHOW_FRAME_REF
  result = OtxDrvGetAttributeValue(hBurstDevice, OTX_ATTR_BURST_GET_FRAME_REF_DATA, &FrameRef, sizeof(FrameRef), &ui32); DemoCheckRc(hBurstDevice, result);
  // just skip if we wrapped
  if (FrameRef.m_FrameCounter > nLastFrame) {
    DemoLogf("Burst FRef:: %d fps  Cntr:%08x%08x  Time:%08x%08x ( ",
      nLastFrame == 0 ? 0 : (OTX_UINT32) (FrameRef.m_FrameCounter - nLastFrame),
      (OTX_UINT32) (FrameRef.m_FrameCounter >> 32),
      (OTX_UINT32) (FrameRef.m_FrameCounter),
      FrameRef.m_FrameDateTime.sUint32.nHigh,
      FrameRef.m_FrameDateTime.sUint32.nLow
    );
    DemoLogf(")\n");
    DemoLogf("\n");
  }
  nLastFrame = FrameRef.m_FrameCounter;
#endif  // #ifdef STATS_SHOW_FRAME_REF


  return;
}

// Returns the string representing the device type for a specific device, automatically chooses between logical and physical devices
static const char *DemoGetDeviceTypeString(IN OTX_HANDLE hDevice)
{
  static char szBuf[256];
  OTX_RESULT otxRc;
  
  otxRc = OtxDrvGetAttributeValueSz(hDevice, OTX_ATTR_DEVICE_STRING, szBuf, 256);
  if (!OTX_SUCCESS(otxRc)) {
      DemoLogf("Could not get the OTX_ATTR_DEVICE_STRING attribute for device 0x%X\n", hDevice);
      return "*** Error ***";
  } else {
    return szBuf;
  }
}


// Returns the device no for a specific device
static signed int DemoGetDeviceNo(IN OTX_HANDLE hDevice)
{
  OTX_UINT32 nDevNo;
  if (OTX_SUCCESS(OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDevNo))) {
    return nDevNo;
  } else {
    return -1;
  }
}


/*******************************************************************************

  @func

  Indents according to ms_nIndentLevel, before a DemoLogf()

  @comm

*******************************************************************************/
static void DemoLogIndent()
{
  fprintf(stdout, "%*s", ms_nIndentLevel*2, "");

  if (g_pDemoLogFile) {
    fprintf(g_pDemoLogFile, "%*s", ms_nIndentLevel*2, "");
  }
}



/*******************************************************************************
	main
*******************************************************************************/
int main(int argc, char *argv[])
{
  OTX_INT32 exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char commandChar;

  // Initialize global values
  for(g_nBurstDeviceCnt = 0; g_nBurstDeviceCnt < DEMO_MAX_BURST_DEVICE; g_nBurstDeviceCnt++) {
    g_pszSerialNumber[g_nBurstDeviceCnt] = 0; 
    g_hBurstDevice[g_nBurstDeviceCnt] = OTX_INVALID_HANDLE_VALUE;
  }

  g_nBurstDeviceCnt = 0;

	OsaConsoleInit();

  DemoLogf("BurstStatusOtxDevices, %s, Odin TeleSystems Inc.\n", __DATE__);
  DemoLogf("OTX Test Program.\n");

  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if (g_demoEnv.pszInputStr) {
    OsaConsoleInput(g_demoEnv.pszInputStr);
  }

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
		OsaConsoleExit();
    exit(-1);
  }

  DemoLogf("Log file is  %s\n", g_demoEnv.logFileName);
  DemoLogf("Board number %d\n", g_demoEnv.boardNo);

  // Connect to the Driver
  DemoLogf("Connecting lib...");
  result = OtxDrvConnectLib(); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
  DemoLogf("Lib connected okay\n");

  DemoLogf("\nPress 'h' for help\n");
  DemoLogf("Press 'a' to enumerate OTX devices\n");

  DemoProcessUserCommand('a');
  // Show help menu
  DemoProcessUserCommand('h');

  while (!exitFlag) {
    commandChar = OsaConsoleGetCh();
    exitFlag = DemoProcessUserCommand(commandChar);
  }

  // Clean up
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

